@@ -1,8 +1,12 @@
-"""Access wrappers for data frame members."""
-from typing import Callable, Optional, Sequence, Union
+"""Access data frame/series from context in ``.loc[]``, ``.iloc[]``, ``.assign()``, and others."""
+
+from typing import Any, Callable, ClassVar, Dict, Iterable, Optional, Union, Tuple, Type
 
 import pandas as pd
 
+# The context in which the wrappers might be used
+PdContext = Union[pd.DataFrame, pd.Series]
+
 # Wrappers for attribute, item and operator access
 class WrapperBase:
     """Base class for wrapping attribute, item or operator/method access."""
@@ -15,41 +19,51 @@ class WrapperBase:
         """
         self.name = name
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"<{type(self).__name__} {self.name}>"
 
-    def __call__(self, obj, root_obj: Union[pd.DataFrame,pd.Series]):
+    def __call__(self, obj: Any, root_obj: PdContext) -> Any:
         """Access member of wrapped object.
 
         Parameters
         ----------
         obj
-            Typically the ``~pandas.DataFrame`` or ``~pandas.Series`` to act
-            upon.
+            Anything that can be reached view item-, attribute-, or
+            method-calls from a ``~pandas.DataFrame`` or ``~pandas.Series``.
         root_obj
             The original data frame or series from the context.
+
+        Returns
+        -------
+        member
+            The referenced member of ``obj``.
         """
         raise NotImplementedError("Must be implemented by a sub-class.")
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         return self.__dict__.copy()
 
-    def __setstate__(self, state):
+    def __setstate__(self, state:Dict[str, Any]):
         self.__dict__.update(state)
 
 
 class Attribute(WrapperBase):
     """Wrap ``df.column_name`` or similar access patterns."""
-    def __call__(self, obj, root_obj):
-        """Access attribute of ``obj``.
+    def __call__(self, obj, root_obj: PdContext) -> Any:
+        """Access attribute of wrapped object.
 
         Parameters
         ----------
         obj
-            Typically the ``~pandas.DataFrame`` or ``~pandas.Series`` to act
-            upon.
+            Anything that can be reached view item-, attribute-, or
+            method-calls from a ``~pandas.DataFrame`` or ``~pandas.Series``.
         root_obj
-            The original data frame or series from the context. (Unused)
+            The original data frame or series from the context.
+
+        Returns
+        -------
+        member
+            The referenced member of ``obj``.
         """
         return getattr(obj, self.name)
 
@@ -59,16 +73,21 @@ class Attribute(WrapperBase):
 
 class Item(WrapperBase):
     """Wrap ``df["column_name"]`` or similar access patterns."""
-    def __call__(self, obj, root_obj):
-        """Access attribute of ``obj``.
+    def __call__(self, obj, root_obj: PdContext):
+        """Access item of wrapped object.
 
         Parameters
         ----------
         obj
-            Typically the ``~pandas.DataFrame`` or ``~pandas.Series`` to act
-            upon.
+            Anything that can be reached view item-, attribute-, or
+            method-calls from a ``~pandas.DataFrame`` or ``~pandas.Series``.
         root_obj
-            The original data frame or series from the context. (Unused)
+            The original data frame or series from the context.
+
+        Returns
+        -------
+        member
+            The referenced member of ``obj``.
         """
         return obj[self.name]
 
@@ -107,21 +126,21 @@ class Method(WrapperBase):
         DF["x"].clip(DF["y"].min())
         DF["x"].clip(upper=DF["y"].min())
     """
-    def __init__(self, name: str,
-                 *args, **kwargs):
+    def __init__(self, name: str, *args: Any, **kwargs: Any):
         """
         Parameters
         ----------
         name
-            Operator name, e.g. `__eq__`.
-        *args, **kwargs
-            Optional arguments for the operator, e.g. the second argument for a binary operator.
+            Method or operator name, e.g. `mean` or `__eq__`.
+        args, kwargs
+            (Optional) arguments for the method or operator, e.g. the second
+            argument for a binary operator.
         """
         super().__init__(name)
         self.args = args
         self.kwargs = kwargs
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         arg_reprs = (
             [f"{a!r}" for a in self.args]
             + [f"{k}={a!r}" for k, a in self.kwargs.items()]
@@ -129,7 +148,7 @@ class Method(WrapperBase):
 
         return f"<{type(self).__name__}: {self.name}({', '.join(arg_reprs)})>"
 
-    def __str__(self):
+    def __str__(self) -> str:
         # Use shorter `str` representation for accessors and `repr` for the
         # rest
         fmt_arg = lambda a: str(a) if isinstance(a, AccessorBase) else repr(a)
@@ -139,16 +158,54 @@ class Method(WrapperBase):
             )
         return f".{self.name}({', '.join(arg_strs)})"
 
-    def _wrap_method_arg(self, arg, root_obj):
+    def _evaluate_method_arg(self, arg: Any, root_obj: PdContext):
+        """Evaluatue any ``DF``- or ``S``-based arguments for the method
+        call.
+
+        This is needed to support things like (``DF['x'].mean()`` will be
+        evaluated with ``df``)::
+
+            df.assign(
+              y = DF['x'].clip(low=DF['x'].mean() - 0.5)
+            )
+
+        Parameters
+        ----------
+        arg
+            The method argument.
+        root_obj
+            The dataframe or series to evaluate ``arg`` with.
+
+        Returns
+        -------
+        eval_arg
+            Evaluated method argument if the argument is ``DF`` or ``S``
+            based, else just ``arg``.
+        """
         if isinstance(arg, AccessorBase):
             return arg(root_obj)
         return arg
 
-    def __call__(self, obj, root_obj):
+    def __call__(self, obj: Any, root_obj: PdContext) -> Any:
+        """Call method ``self.name`` on ``obj``.
+
+        Parameters
+        ----------
+        obj
+            Anything that can be reached view item-,attribute- or
+            method-calls from a ``~pandas.DataFrame`` or ``~pandas.Series``.
+        root_obj
+            The original data frame or series from the context.
+
+        Returns
+        -------
+        result
+            The result of the method or operator call.
+        """
         op_meth = getattr(obj, self.name)
         return op_meth(
-            *[self._wrap_method_arg(arg, root_obj) for arg in self.args],
-            **{k: self._wrap_method_arg(arg, root_obj) for k, arg in self.kwargs.items()}
+            *[self._evaluate_method_arg(arg, root_obj) for arg in self.args],
+            **{k: self._evaluate_method_arg(arg, root_obj) for k, arg in self.kwargs.items()}
         )
 
 
@@ -210,52 +267,55 @@ def _add_dunder_operators(cls):
     return cls
 
 
-def _get_obj_attr_doc(obj: type, attr: str):
+def _get_obj_attr_doc(obj_or_class: Any, attr: str):
+    """Get doc-string for attribute ``attr`` of ``obj_or_class`` if it exists."""
     if isinstance(attr, str):
-        a = getattr(obj, attr, None)
+        a = getattr(obj_or_class, attr, None)
         if a:
             return a.__doc__
     return None
 
 
 class AccessorBase:
-    wrapped_cls = None
+    """Abstract base-class for DataFrame and Series context accessors."""
+    wrapped_cls: ClassVar[Type] = type('NotABaseOfAnything', (), {})
     def __init__(self,
-                 levels:Union[Sequence[Callable], None]=None):
+                 levels: Optional[Iterable[WrapperBase]]=None):
         """
         Parameters
         ----------
         levels:
-            Sequence of callables to extract attributes from data frames or series.
+            Iterable of callables to extract attributes from data frames or series.
         """
-        self._levels = levels or ()
+        self._levels: Tuple[WrapperBase, ...] = ()
+        if levels is not None:
+            self._levels = tuple(levels)
 
-        d = self._get_doc()
-        if d:
-            self.__doc__ = d
+        # Update the doc-string if possible
+        self.__doc__ = self._get_doc()
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         return self.__dict__.copy()
 
-    def __setstate__(self, state):
+    def __setstate__(self, state: Dict[str, Any]):
         self.__dict__.update(state)
 
-    def _get_doc(self):
-        return None
+    def _get_doc(self) -> Optional[str]:
+        return type(self).__doc__
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f"<{type(self).__name__} {'.'.join(repr(l) for l in self._levels)}>"
 
-    def __str__(self):
+    def __str__(self) -> str:
         return ''.join(str(l) for l in self._levels)
 
-    def __getattr__(self, name):
+    def __getattr__(self, name: str) -> 'AccessorBase':
         return type(self)(self._levels + (Attribute(name),))
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: str) -> 'AccessorBase':
         return type(self)(self._levels + (Item(key),))
 
-    def _operator_proxy(self, op_name):
+    def _operator_proxy(self, op_name: str) -> Callable:
         """Generate proxy function for built-in operators.
 
         Used by :func:`_add_dunder_operators`
@@ -264,7 +324,7 @@ class AccessorBase:
             return type(self)(self._levels + (Method(op_name, *args, **kwargs),))
         return op_wrapper
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args: Any, **kwargs: Any) -> Union[pd.DataFrame, pd.Series, Any]:
         # Heuristic: Assume the selector is applied if exactly one DataFrame
         # or Series argument is passed.
         if len(args) == 1 and isinstance(args[0], self.wrapped_cls):
@@ -279,7 +339,7 @@ class AccessorBase:
 
 @_add_dunder_operators # This is necessary to overload all dunder operators.
 class DataframeAccessor(AccessorBase):
-    """Build callable for column access and operators.
+    """Build callable to access columns and operators.
 
     Use the global instance like::
 
@@ -309,24 +369,28 @@ class DataframeAccessor(AccessorBase):
     >>> df
     """
     wrapped_cls = pd.DataFrame
-    def _get_doc(self):
-        doc = None
+    def _get_doc(self) -> Optional[str]:
+        doc = super()._get_doc()
         # Assume DataFrame-level function for 1-level accessor
         if len(self._levels) == 1 and isinstance(self._levels[-1].name, str):
-            doc = _get_obj_attr_doc(self.wrapped_cls, self._levels[-1].name)
+            doc = _get_obj_attr_doc(self.wrapped_cls, self._levels[-1].name) or doc
         # Check for typed Series accessors for 3+-level accessor
         elif len(self._levels) >= 3 and self._levels[-2].name in ('dt', 'str'):
-            doc = _get_obj_attr_doc(
-                getattr(pd.Series, self._levels[-2].name),
-                self._levels[-1].name,
+            doc = (
+                _get_obj_attr_doc(
+                    getattr(pd.Series, self._levels[-2].name),
+                    self._levels[-1].name,
                 )
+                or doc
+            )
         # Check for Series-level function for 2+-level accessor
         elif len(self._levels) > 1:
-            doc = _get_obj_attr_doc(pd.Series, self._levels[-1].name)
+            doc = _get_obj_attr_doc(pd.Series, self._levels[-1].name) or doc
 
         return doc
 
 
+
 @_add_dunder_operators # This is necessary to overload all dunder operators.
 class SeriesAccessor(AccessorBase):
     """Build callable for series attributes and operators.
@@ -352,16 +416,19 @@ class SeriesAccessor(AccessorBase):
     >>> s[S <= 2]
     """
     wrapped_cls = pd.Series
-    def _get_doc(self):
-        doc = None
+    def _get_doc(self) -> Optional[str]:
+        doc = super()._get_doc()
         # Assume Series-level function for 1-level accessor
         if len(self._levels) == 1 and isinstance(self._levels[-1].name, str):
-            doc = _get_obj_attr_doc(self.wrapped_cls, self._levels[-1].name)
+            doc = _get_obj_attr_doc(self.wrapped_cls, self._levels[-1].name) or doc
         # Check for typed Series accessors
         elif len(self._levels) > 1 and self._levels[0].name in ('dt', 'str'):
-            doc = _get_obj_attr_doc(
-                getattr(pd.Series, self._levels[0].name),
-                self._levels[-1].name,
+            doc = (
+                _get_obj_attr_doc(
+                    getattr(pd.Series, self._levels[0].name),
+                    self._levels[-1].name,
                 )
+                or doc
+            )
 
         return doc
